#include "Collisions.h"

#include "ObjectStruct.h"
#include "Debug.h"
#include "LevelManager.h"
#include "MathUtils.h"
#include "GGAnimationTypes.h"
#include "ObjectManager.h"
#include "PlayerManager.h"
#include "ScrollManager.h"
#include "VDPInfo.h"


u16 IsTileSolid_Tile(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, x + (y * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile) & TILE_SOLID;
}

u16 IsTileSolid_Virtual(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap,  VIRTUAL_TO_TILE(x) + (VIRTUAL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile) & TILE_SOLID;
}

u16 IsTileSolid_Pixel(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, PIXEL_TO_TILE(x) + (PIXEL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile) & TILE_SOLID;
}

u16 IsTileSemiSolid_Tile(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, x + (y * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile) & TILE_SEMI_SOLID;
}

u16 IsTileSemiSolid_Virtual(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap,  VIRTUAL_TO_TILE(x) + (VIRTUAL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile) & TILE_SEMI_SOLID;
}

u16 IsTileSemiSolid_Pixel(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, PIXEL_TO_TILE(x) + (PIXEL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile) & TILE_SEMI_SOLID;
}

u16 IsTileSolidOrSemiSolid_Virtual(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap,  VIRTUAL_TO_TILE(x) + (VIRTUAL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    u16 itemValue = ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile);
    return (itemValue & TILE_SOLID) || (itemValue & TILE_SEMI_SOLID);
}

u16 IsTileSolidOrSemiSolid_Pixel(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap,  PIXEL_TO_TILE(x) + (PIXEL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    u16 itemValue = ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile);
    return (itemValue & TILE_SOLID) || (itemValue & TILE_SEMI_SOLID);
}

u16 GetTile_Virtual(u16 x, u16 y)
{
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, VIRTUAL_TO_TILE(x) + (VIRTUAL_TO_TILE(y) * currentForegroundMap->width));
}

u16 GetTile_Pixel(u16 x, u16 y)
{
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, PIXEL_TO_TILE(x) + (PIXEL_TO_TILE(y) * currentForegroundMap->width));
}

u16 GetTile_Tile(u16 x, u16 y)
{
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, x + (y * currentForegroundMap->width));
}

u16 IsTileFlippedHorizontally_Tile(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, VIRTUAL_TO_TILE(x) + (VIRTUAL_TO_TILE(y) * currentForegroundMap->width));
    return tile & 0x800;
}

u16 GetTileType_Virtual(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, VIRTUAL_TO_TILE(x) + (VIRTUAL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile);
}

u16 GetTileType_Pixel(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, PIXEL_TO_TILE(x) + (PIXEL_TO_TILE(y) * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile);
}

u16 GetTileType_Tile(u16 x, u16 y)
{
    u16 tile = ARRAY_ITEM_VALUE(currentForegroundMap->tilemap, x + (y * currentForegroundMap->width)) & TILEINDEX_BITMASK;
    return ARRAY_ITEM_VALUE(currentForegroundMap->tilesetAttributes, tile);
}

u16 OutOfBounds(s16 x, s16 y)
{
    return (x < 0 ||
            y < 0 ||
            x > TILE_TO_VIRTUAL(currentForegroundMap->width) ||
            y > TILE_TO_VIRTUAL(currentForegroundMap->height));
}

u16 ObjectIsOffScreen(s16 x, s16 y)
{
    return (x < foregroundScrollData.scrollx_vdp ||
            y < foregroundScrollData.scrolly_vdp ||
            x > foregroundScrollData.scrollx_vdp + SCREEN_WIDTH)||
            y > (foregroundScrollData.scrolly_vdp + SCREEN_HEIGHT);

}

u16 ObjectIsOnScreen(const ObjectStruct* object)
{
    return ((object->spritedef.posx + object->animation->width >= 0) &&
            (object->spritedef.posy + object->animation->height >= 0) &&
            (object->spritedef.posx <= SCREEN_WIDTH) &&
            (object->spritedef.posy <= SCREEN_HEIGHT));
}

u16 ActiveAreaIsOnScreen(const ObjectStruct* object)
{
    return !((object->activeAreaRectLeft   > foregroundScrollData.scrollx_vdp + SCREEN_WIDTH) ||
             (object->activeAreaRectTop    > foregroundScrollData.scrolly_vdp + SCREEN_HEIGHT) ||
             (object->activeAreaRectRight  < foregroundScrollData.scrollx_vdp) ||
             (object->activeAreaRectBottom < foregroundScrollData.scrolly_vdp));
}

u16 SpriteIsOnScreen(s16 x, s16 y, s16 padding)
{
    return (x >= 0 - padding &&
            y >= 0 - padding &&
            x <= SCREEN_WIDTH + padding &&
            y <= SCREEN_HEIGHT + padding);

}

u16 PointInObjectRect(s16 x, s16 y, const ObjectStruct* object)
{
    return ((x >= object->x + object->rectLeft) && 
            (x <= object->x + object->rectRight) &&
            (y >= object->y + object->rectTop) &&
            (y <= object->y + object->rectBottom));
}


u16 PointSquareCollision(const ObjectStruct* firstObject, const ObjectStruct* secondObject)
{
    return ((firstObject->x >= secondObject->x + secondObject->rectLeft) && 
            (firstObject->x <= secondObject->x + secondObject->rectRight) &&
            (firstObject->y >= secondObject->y + secondObject->rectTop) &&
            (firstObject->y <= secondObject->y + secondObject->rectBottom));
}


u16 SquareSquareCollision(const ObjectStruct* firstObject, const ObjectStruct* secondObject)
{
    return !(firstObject->x + firstObject->rectLeft   > secondObject->x + secondObject->rectRight ||
             firstObject->x + firstObject->rectRight  < secondObject->x + secondObject->rectLeft ||
             firstObject->y + firstObject->rectTop    > secondObject->y + secondObject->rectBottom ||
             firstObject->y + firstObject->rectBottom < secondObject->y + secondObject->rectTop);
}

