#ifndef COLLISIONS_INCLUDE_H
#define COLLISIONS_INCLUDE_H

#include <genesis.h>

#include "ObjectStruct.h"

// tile types (8 types max)
#define TILE_NONE                       0x00 // 0b0000000000000000
#define TILE_SOLID                      0x01 // 0b0000000000000001
#define TILE_SLOPE                      0x02 // 0b0000000000000010
#define TILE_SEMI_SOLID                 0x04 // 0b0000000000000100

// slope types (256 types max)
#define TILE_45DEGREES_SLOPE            0x10 // 0b0000000100000000
#define TILE_30DEGREES_SLOPE_BOTTOM     0x20 // 0b0000001000000000
#define TILE_30DEGREES_SLOPE_TOP        0x30 // 0b0000001100000000

#define IS_FLIPPED_HORIZONTALLY(x) (x & 0x800)

#define TILEINDEX_BITMASK 0x07FF // 0b0000 0111 1111 1111

u16 GetTile_Virtual(u16 x, u16 y);
u16 GetTile_Pixel(u16 x, u16 y);
u16 GetTile_Tile(u16 x, u16 y);

u16 GetTileType_Virtual(u16 x, u16 y);
u16 GetTileType_Pixel(u16 x, u16 y);
u16 GetTileType_Tile(u16 x, u16 y);

u16 IsTileSolidOrSemiSolid_Virtual(u16 x, u16 y);
u16 IsTileSolidOrSemiSolid_Pixel(u16 x, u16 y);

u16 IsTileFlippedHorizontally_Tile(u16 x, u16 y);

u16 IsTileSolid_Virtual(u16 x, u16 y);
u16 IsTileSolid_Pixel(u16 x, u16 y);
u16 IsTileSolid_Tile(u16 x, u16 y);

u16 IsTileSemiSolid_Virtual(u16 x, u16 y);
u16 IsTileSemiSolid_Pixel(u16 x, u16 y);
u16 IsTileSemiSolid_Tile(u16 x, u16 y);

u16 OutOfBounds(s16 x, s16 y);
u16 ObjectIsOffScreen(s16 x, s16 y);
u16 ObjectIsOnScreen(const ObjectStruct* object);
u16 ActiveAreaIsOnScreen(const ObjectStruct* object);
u16 SpriteIsOnScreen(s16 x, s16 y, s16 padding);


typedef u16 (*CollisionFunction)(const ObjectStruct* firstObject, const ObjectStruct* secondObject);

u16 PointInObjectRect(s16 x, s16 y, const ObjectStruct* object);

u16 PointSquareCollision(const ObjectStruct* firstObject, const ObjectStruct* secondObject);
u16 SquareSquareCollision(const ObjectStruct* firstObject, const ObjectStruct* secondObject);

#endif
