#include "DoorManager.h"

#include "generated\DebugSettings.h"
#include "PlayerManager.h"
#include "FadeManager.h"
#include "LevelManager.h"
#include "ScrollManager.h"
#include "StateManager.h"

#define DOORMANAGER_STATE_WAITING   0
#define DOORMANAGER_STATE_EXITING   1
#define DOORMANAGER_STATE_ENTERING  2

#define EXITING_TIME    64
#define ENTERING_TIME   64

typedef void (*DoorManagerFunctionType)(void);

u16 doorState = DOORMANAGER_STATE_WAITING;
u16 updateTimer = 0;
DoorManagerFunctionType doorManagerUpdate;

const Door* lastDoor = NULL;

void DoorManager_UpdateWaiting();
void DoorManager_UpdateExiting();
void DoorManager_UpdateEntering();

void DoorManager_StartExit();
void DoorManager_StartEntering();
void DoorManager_StartWaiting();


void DoorManager_Update()
{
    doorManagerUpdate();
}

void DoorManager_Init()
{
    DoorManager_StartEntering();
}

void DoorManager_ExitLevel()
{
    LevelManager_SetLevel(currentLevel->nextlevel);
    DoorManager_StartExit();
}

void DoorManager_StartWaiting()
{
    doorManagerUpdate = DoorManager_UpdateWaiting;
}

u16 TouchesDoor(const Door* door, const ObjectStruct* object)
{
    return !(door->left     > object->x + object->rectRight ||
             door->right    < object->x + object->rectLeft ||
             door->top      > object->y + object->rectBottom ||
             door->bottom   < object->y + object->rectTop);
}


void DoorManager_UpdateWaiting()
{
    if (playerObject->healthPoints == 0)
    {
        return;
    }

    if (TouchesDoor(&currentForegroundMap->door, playerObject))
    {
        u16 enterDoor = 0;

        if ((currentForegroundMap->door.doorType == DOOR_LEFT) && 
            (playerData.playerState == PLAYERSTATE_WALK || debugSettings.debugmovement == 1))
        {
            PlayerManager_StartAutomatedWalkLeft();
            enterDoor = 1;
        }
        else if ((currentForegroundMap->door.doorType == DOOR_RIGHT) && 
                 (playerData.playerState == PLAYERSTATE_WALK || debugSettings.debugmovement == 1))
        {
            PlayerManager_StartAutomatedWalkRight();
            enterDoor = 1;
        }
        else if ((currentForegroundMap->door.doorType == DOOR_BOTTOM) && 
                 (playerData.playerState == PLAYERSTATE_FALL))
        {
            PlayerManager_StartAutomatedFall();
            enterDoor = 1;
        }

        if (enterDoor == 1)
        {
            lastDoor = &currentForegroundMap->door;
            LevelManager_SetLevel(currentLevel->nextlevel);
            DoorManager_StartExit();
        }
    }
}

void DoorManager_StartExit()
{
    doorManagerUpdate = DoorManager_UpdateExiting;
    updateTimer = EXITING_TIME;
    FadeManager_FadeOut();
}

void DoorManager_UpdateExiting()
{
    if (updateTimer > 0)
    {
        updateTimer--;

        if (updateTimer == 0)
        {
            StateManager_ChangeToState(LevelManager_StartLevel);
        }
    }
}

void DoorManager_StartEntering()
{
    doorManagerUpdate = DoorManager_UpdateEntering;
    updateTimer = ENTERING_TIME;
    FadeManager_FadeIn();

    if (lastDoor != NULL)
    {
        if (lastDoor->doorType == DOOR_RIGHT)
        {
            PlayerManager_StartAutomatedWalkRight();
        }
        else if (lastDoor->doorType == DOOR_LEFT)
        {
            PlayerManager_StartAutomatedWalkLeft();
        }
        else 
        {
            PlayerManager_StartAutomatedFall();
        }
    }
    else
    {
        PlayerManager_StartAutomatedWalkRight();
    }
}

void DoorManager_UpdateEntering()
{
    if (isEndingScreen)
    {
        DoorManager_StartWaiting();
        PlayerManager_Happy();
        return;
    }

    if (updateTimer > 0)
    {
        updateTimer--;

        if (updateTimer == 0)
        {
            DoorManager_StartWaiting();
            
            if (lastDoor != NULL)
            {
                if (lastDoor->doorType == DOOR_RIGHT || 
                    lastDoor->doorType == DOOR_LEFT)
                {
                    PlayerManager_ReturnControlStartIdle();
                }
                else 
                {
                    PlayerManager_ReturnControlStartFall();
                }
            }
            else
            {
                PlayerManager_ReturnControlStartIdle();
            }
        }
    }
}

