#include "GGAnimationDraw.h"
#include "Debug.h"
#include "vdp_spr.h"
#include "ObjectStruct.h"
#include "ObjectManager.h"
#include "exported\ExportedGGAnimations.h"
#include "VDP_Extra.h"


void GGAnimation_Draw(s16 x, 
                    s16 y, 
                    const GGAnimation* animation, 
                    u16 frameNumber, 
                    u16 direction,
                    u16 palette)
{
    u16 loop = 0;
    u16 tileInfo = TILE_ATTR_FULL(palette, 0, 0, direction, animation->vdpInfo->startIndex);
    const GGFrame* frame = *(animation->frames + frameNumber);
    const GGSprite* sprite = *(frame->sprites);

    for (loop = frame->numSprites; loop > 0; loop--)
    {
        if (spriteDrawIndex >= MAX_SPRITE) return;

        if (direction == DIRECTION_RIGHT)
        {
            currentSpriteDef->posx = x + sprite->xoffset;
        }
        else
        {
            currentSpriteDef->posx = x + animation->width - (sprite->xoffset + (sprite->tilewidth << 3));
        }

        currentSpriteDef->posy = y + sprite->yoffset;
        currentSpriteDef->tile_attr = tileInfo + sprite->VDPTileOffset;
        currentSpriteDef->size = sprite->sgdkSpriteSize;
        spriteDrawIndex++;
        currentSpriteDef->link = spriteDrawIndex;
        currentSpriteDef++;

        sprite++;
    }
}

void GGAnimation_DrawObject(ObjectStruct* object)
{
    u16 loop = 0;
    u16 tileInfo = TILE_ATTR_FULL(object->palette, 0, 0, object->direction, object->animation->vdpInfo->startIndex);
    const GGFrame* frame = *(object->animation->frames + object->animationFrame);
    const GGSprite* sprite = *(frame->sprites);

    for (loop = frame->numSprites; loop > 0; loop--)
    {
        if (spriteDrawIndex >= MAX_SPRITE) return;

        if (object->direction == DIRECTION_RIGHT)
        {
            currentSpriteDef->posx = object->spritedef.posx + sprite->xoffset;
        }
        else
        {
            currentSpriteDef->posx = object->spritedef.posx + object->animation->width - (sprite->xoffset + (sprite->tilewidth << 3));
        }

        currentSpriteDef->posy = object->spritedef.posy + sprite->yoffset;
        currentSpriteDef->tile_attr = tileInfo + sprite->VDPTileOffset;
        currentSpriteDef->size = sprite->sgdkSpriteSize;
        spriteDrawIndex++;
        currentSpriteDef->link = spriteDrawIndex;
        currentSpriteDef++;

        sprite++;
    }
}

// The difference between this and Animation_Draw is that the offset to choose the
// next sprite VDP tiles to draw is relative to the VDPInfo of the tiles of the
// frame that has been copied to the VDP directly. Animation_Draw uses the offset
// relative to the tile data of the entire animation copied to the VDP.
void GGAnimation_DrawPlayer(s16 x, 
                          s16 y, 
                          const GGAnimation* animation, 
                          const VDPInfo* vdpInfo, 
                          u16 frameNumber, 
                          u16 direction,
                          u16 palette)
{
    u16 loop = 0;
    u16 vpdOffset = 0;
    u16 tileInfo = TILE_ATTR_FULL(palette, 0, 0, direction, vdpInfo->startIndex);
    const GGFrame* frame = *(animation->frames + frameNumber);
    const GGSprite* sprite = *(frame->sprites);

    for (loop = frame->numSprites; loop > 0; loop--)
    {
        if (spriteDrawIndex >= MAX_SPRITE) return;

        if (direction == DIRECTION_RIGHT)
        {
            currentSpriteDef->posx = x + sprite->xoffset;
        }
        else
        {
            currentSpriteDef->posx = x + animation->width - (sprite->xoffset + (sprite->tilewidth << 3));
        }

        currentSpriteDef->posy = y + sprite->yoffset;
        currentSpriteDef->tile_attr = tileInfo + vpdOffset;
        currentSpriteDef->size = sprite->sgdkSpriteSize;
        spriteDrawIndex++;
        currentSpriteDef->link = spriteDrawIndex;
        currentSpriteDef++;

        vpdOffset += (sprite->tilewidth * sprite->tileheight);
        sprite++;
    }
}

