#ifndef OBJECTSTRUCT_INCLUDE_H
#define OBJECTSTRUCT_INCLUDE_H

#include <genesis.h>
#include <vdp_spr.h>

#include "GGAnimationTypes.h"

#define OBJECTSTATE_ACTIVE    0
#define OBJECTSTATE_INACTIVE  1

// object types.
#define TYPE_PLAYER_PROJECTILE  0
#define TYPE_EFFECT             1
#define TYPE_ENEMY              2
#define TYPE_ENEMY_PROJECTILE   3
#define TYPE_PICKUP             4
#define TYPE_PLATFORM           5
#define MAX_OBJECT_TYPES        6
// specially treated object types.
#define TYPE_NONE               6
#define TYPE_PLAYER             7


typedef struct object_struct
{
    s16 objectStreamerId;
    struct object_struct* nextVisibleObjectOfSameType;
    SpriteDef spritedef;
    s16 x;
    s16 y;
    u16 direction;
    u16 palette;
    void (*updateFunction)(struct object_struct* objectStruct);
    void (*destroyFunction)(struct object_struct* objectStruct);
    void (*drawFunction)(struct object_struct* objectStruct);
    void (*collisionFunction)(struct object_struct* objectStruct, struct object_struct* collidedObject);
    u16 objectState;
    u16 aiState;
    const GGAnimation* animation;
    s16 speedx;
    s16 speedy;
    u16 lifetime; // one tick every frame. 60 ticks every second. 3600 ticks every minute. Max 18.2 minutes. only for temporary objects.
    u16 objectType;
    s16 rectLeft;
    s16 rectRight;
    s16 rectTop;
    s16 rectBottom;
    s16 activeAreaRectLeft;
    s16 activeAreaRectRight;
    s16 activeAreaRectTop;
    s16 activeAreaRectBottom;
    s16 healthPoints;
    u16 damage;
    u16 visible;
    u16 animationFrame;
    u16 animationSubFrameCounter;
    u16 invulnerableTimer;
    s32 extra1;
    s32 extra2;
    s32 extra3;
    s32 extra4;
} object_struct;

typedef struct object_struct ObjectStruct;

typedef ObjectStruct* (*ObjectCreateFunctionType) (s16 x, s16 y);
typedef void (*ObjectFunctionType)(ObjectStruct* objectStruct);
typedef void (*ObjectCollisionFunctionType)(ObjectStruct* objectStruct, ObjectStruct* collidedObject);

#endif
