#ifndef PLAYER_INCLUDE_H
#define PLAYER_INCLUDE_H

#include "ObjectStruct.h"
#include "VDPInfo.h"

#define PLAYER_PIXEL_HEIGHT 40

// player states
#define PLAYERSTATE_IDLE                0
#define PLAYERSTATE_WALK                1
#define PLAYERSTATE_JUMP                2
#define PLAYERSTATE_FALL                3
#define PLAYERSTATE_HAPPY               5
#define PLAYERSTATE_AUTOMATED           6

typedef struct
{
    u16 playerState;
    s16 playerLevelStartX;
    s16 playerLevelStartY;
    s16 oldPositionX;
    s16 oldPositionY;
    u16 order;
    u16 freezeTimer;
    void (*onfreezeTimerDone)();
    u16 invincibilityTimer;
    u16 onGround;
    u16 disablePlayer;
    ObjectStruct* platform;
} PlayerDataStruct;

extern u16 isEndingScreen;

// externs
extern ObjectStruct* playerObject;
extern VDPInfo playerVDPInfo;
extern PlayerDataStruct playerData;

// setup at various times.
void PlayerManager_SetupForGame();
void PlayerManager_SetupForLevel();
void PlayerManager_SetupLife();
void PlayerManager_InitPosition(s16 x, s16 y);

// affects player state
void PlayerManager_Jump();
void PlayerManager_Happy();
void PlayerManager_StartDebugMovement();
void PlayerManager_EndDebugMovement();

// handle collisions for moving platforms
void PlayerManager_CallOnPhysicsCollisionFunction();

// scripting helper functions.
void PlayerManager_StartAutomatedWalkLeft();
void PlayerManager_StartAutomatedWalkRight();
void PlayerManager_StartAutomatedFall();
void PlayerManager_ReturnControlStartIdle();
void PlayerManager_ReturnControlStartFall();

// copies animation frame to vdp during vblank
void PlayerManager_UpdateVDP();

#endif
