#include "PlatformVertical.h"

#include "PlatformBase.h"
#include "ObjectCommon.h"

void PlatformHorizontalUpdate(ObjectStruct* object);

ObjectStruct* PlatformHorizontalCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PLATFORM,
                                                      PlatformHorizontalUpdate, 
                                                      PlatformBaseDestroy, 
                                                      GGAnimation_DrawObject,
                                                      PlatformBaseCollide);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("platform could not be created!");
#endif
        return NULL;
    }

    object->x                           = x;
    object->y                           = y; // y spawn position is at the bottom of the sprite
    object->speedx                      = 2;
    object->speedy                      = 0;
    object->healthPoints                = 1;
    object->animation                   = &Platform;
    object->rectLeft                    = PIXEL_TO_VIRTUAL(0);
    object->rectRight                   = PIXEL_TO_VIRTUAL(Platform.width);
    object->rectTop                     = PIXEL_TO_VIRTUAL(0);
    object->rectBottom                  = PIXEL_TO_VIRTUAL(Platform.height >> 1);
    object->visible                     = 1;
    object->animationFrame              = 0;
    object->animationSubFrameCounter    = 0;
    object->extra1                      = PIXEL_TO_VIRTUAL(24);
    object->extra2                      = 0;
    object->extra3                      = object->x; // original x
    object->extra4                      = object->y; // original y
    object->direction                   = 0;
    object->palette                     = PAL0;
    ComputeStandardActiveArea(object);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("PlatformVerticalCreate");
#endif
    
    return object;
}

void PlatformHorizontalUpdate(ObjectStruct* object)
{
    object->extra2 += object->speedx;
    if (mathutils_abs(object->extra2) > mathutils_abs(object->extra1))
    {
        object->speedx = -object->speedx; 
    }

    object->spritedef.posx = VIRTUAL_TO_PIXEL(object->extra3 + object->extra2) - foregroundScrollData.scrollx_vdp;
    object->spritedef.posy = VIRTUAL_TO_PIXEL(object->extra4) - foregroundScrollData.scrolly_vdp;

    object->x = object->extra3 + object->extra2;
    object->y = object->extra4;
}

