# helper script that generates .h and .c files from tiled map files

SECTION_PIXEL_SIZE = 16
SECTION_TILE_SIZE = SECTION_PIXEL_SIZE / 8

import sys
import os
import time
import xml.etree.ElementTree as ET
from collections import defaultdict

parts = []
numParts = 0

#changing the destination folder requires updating the SGDK makefile too
destination_folder = 'exported\\complexanimations'
destination_path = os.path.realpath('.') + "\\" + destination_folder

def importcomplexanimationfile(filename):
    global numParts
    global parts

    parts = []
    numParts = 0

    tree = ET.parse(filename)
    root = tree.getroot()

    for part in root:
        if part.get('asset') != None:
            d = {}
            d['frame'] = part.get('frame')
            d['asset'] = part.get('asset')
            d['x'] = part.get('x')
            d['y'] = part.get('y')
            parts.append(d)
            numParts = numParts + 1

    return

def exportcfile(outputname, destinationfilename):

    exported_c = open(destinationfilename, "w")
    exported_c.write("// **************************************************** \n")
    exported_c.write("// This file is automatically generated by\n")
    exported_c.write("// export_complex_animations.py\n")
    exported_c.write("// **************************************************** \n\n")    
    
    exported_c.write('#include "' + outputname + '.h"\n\n')

    for part in parts:
        exported_c.write('extern const GGAnimation ' + part['asset'] + ';\n')
    exported_c.write('\n')

    exported_c.write('const ComplexAnimationPart ' + outputname + 'Parts[' + str(numParts) + '] = \n')
    exported_c.write('{\n')

    for part in parts:
        exported_c.write('    {\n')
        exported_c.write('        &' + part['asset'] + ',\n')
        exported_c.write('        ' + part['x'] + ',\n')
        exported_c.write('        ' + part['y'] + ',\n')
        exported_c.write('        ' + part['frame'] + '\n')
        exported_c.write('    },\n')

    exported_c.write('}; \n\n')

    exported_c.write('\n')
    exported_c.write('const ComplexAnimation ' + outputname + ' = \n')
    exported_c.write('{\n')
    exported_c.write('    ' + outputname + 'Parts,\n')
    exported_c.write('    ' + str(numParts) + '\n')
    exported_c.write('}; \n\n')


    exported_c.close()

def exporthfile(outputname, destinationfilename):
    exported_h = open(destinationfilename, "w")

    exported_h.write("// **************************************************** \n")
    exported_h.write("// This file is automatically generated by\n")
    exported_h.write("// export_complex_animations.py\n")
    exported_h.write("// **************************************************** \n\n")
    exported_h.write('#ifndef ' + outputname.upper() + '_COMPLEX_ANIMATION_INCLUDE_H\n')
    exported_h.write('#define ' + outputname.upper() + '_COMPLEX_ANIMATION_INCLUDE_H\n\n')
    exported_h.write('#include "..\..\ComplexAnimationTypes.h"\n\n')
    exported_h.write('#include <genesis.h>\n\n')
    exported_h.write('extern const ComplexAnimation ' + outputname + ';\n')
    exported_h.write('\n#endif\n')
    exported_h.close()  



from glob import glob
print "************** Exporting Tiled Maps To Code **************"
filenames = glob('..\gamedata\complexanimations\*.anim')
    
includes=[]
    
for filename in filenames:
    sourcefilename = os.path.realpath('.') + "\\" + filename
    
    outputname = os.path.splitext(filename)[0]
    outputname = os.path.basename(outputname)   
    destinationcfilename = os.path.realpath('.') + '\\' + destination_folder + '\\' + outputname + '.c'
    destinationhfilename = os.path.realpath('.') + '\\' + destination_folder + '\\' + outputname + '.h'

    includes.append(outputname + ".h")
    
    docopy = False
    
    if (os.path.exists(destinationcfilename) == False):
        #print destinationcfilename + " is missing. Copying"
        docopy = True
    
    if docopy == False:
        sourcetime = os.path.getmtime(sourcefilename)
        destinationtime = os.path.getmtime(destinationcfilename)

        if sourcetime > destinationtime:
            #print "sourcefilename is newer than destination filename"
            docopy = True

    if docopy == True:
        print "Exporting " + filename + " to " + destination_folder

        if not os.path.exists(destination_path):
            os.makedirs(destination_path)
    
        importcomplexanimationfile(filename)
        exportcfile(outputname, destinationcfilename)
        exporthfile(outputname, destinationhfilename)
    
    
exported_h = open("exported\\ExportedComplexAnimations.h", "w")
exported_h.write("// **************************************************** \n")
exported_h.write("// This file is automatically generated by\n")
exported_h.write("// export_complex_animations.py\n")
exported_h.write("// **************************************************** \n\n")
exported_h.write('#ifndef EXPORTED_COMPLEX_ANIMATIONS_H\n')
exported_h.write('#define EXPORTED_COMPLEX_ANIMATIONS_H\n\n')

for include in includes:
    exported_h.write('#include "complexanimations\\' + include + '"\n')

exported_h.write('\n#endif\n')
exported_h.close()
    
