# helper script that generates .h and .c files from tiled map files

SECTION_PIXEL_SIZE = 16
SECTION_TILE_SIZE = SECTION_PIXEL_SIZE / 8

import sys
import os
import time
import xml.etree.ElementTree as ET
from collections import defaultdict

#changing the destination folder requires updating the SGDK makefile too
destination_folder = 'exported\\maps'
destination_path = os.path.realpath('.') + "\\" + destination_folder

tilesetbitmapfilenames = set()

sections = {}
playerStartX = 0
playerStartY = 0
sections_count = 0
mapwidth = 0
mapheight = 0
maptype = 'none'
vdpwidth = 0
vdpheight = 0
tilemapdata = 0
doorcontents = ""
objectgroup = None
vpdassetlistdict = {}
tilesetfilename = ""

def getshiftvalue(value):
    shiftvalue = 0
    while value > 1:
        shiftvalue = shiftvalue + 1
        value = int(value) >> 1
    return shiftvalue

def gettilesetfilename(tilesetnode):
    global tilesetfilename
    sourcefilename = tilesetnode.get('source')

    currentdirectory = os.getcwd()
    os.chdir('..\\gamedata\\tilesets')
    
    tree = ET.parse(sourcefilename)
    root = tree.getroot()
    imagenode = root.find('image')
    tilesetfilename = imagenode.get('source')
    tilesetfilename = tilesetfilename.rsplit('.', 1)[0]
    os.chdir( currentdirectory)
    

    

def importmap(filename):
    global mapwidth
    global mapheight
    global tilemapdata
    global doorcontents
    global objectgroup

    tree = ET.parse(filename)
    root = tree.getroot()
    mapwidth = int(root.get('width'))
    mapheight = int(root.get('height'))
    tilemapdata = root.find('layer').find('data').text
    tilemapdata = tilemapdata.split(',')
    gettilesetfilename(root.find('tileset'))

    doorcontents = "{TILE_TO_VIRTUAL(0), TILE_TO_VIRTUAL(0), TILE_TO_VIRTUAL(0), TILE_TO_VIRTUAL(0), 0}"

    objectgroup = root.find('objectgroup')
    if objectgroup != None:
        # this will only really export the last door finds.
        for child in objectgroup:
            name = child.get('name')
            if name is not None:
                name = str(name).lower()
                if name == "door":
                    #left, right, top, bottom
                    x = (int)(child.get('x')) >> 3
                    y = (int)(child.get('y')) >> 3
                    width = (int)(child.get('width')) >> 3
                    height = (int)(child.get('height')) >> 3
                    properties = child.find('properties')
                    for property in properties.findall('property'):
                        if property.get('name') == "doortype":
                            doortype = str(property.get('value')).upper()
                    doorcontents = '{TILE_TO_VIRTUAL(' + str(x) + '), TILE_TO_VIRTUAL(' + str(width + x) + '), TILE_TO_VIRTUAL(' + str(y) + '), TILE_TO_VIRTUAL(' + str(y + height) + '), ' + doortype + '}'


def computevdpsettings():
    global maptype
    global vdpwidth
    global vdpheight
    global mapwidth
    global mapheight

    if (mapwidth * mapheight <= 64 * 64) and (mapwidth <= 128 and mapheight <= 128):
        maptype = 'MAPTYPE_VDP'

        if mapwidth <= 32:
            vdpwidth = 32

            if mapheight <= 32:
                vdpheight = 32
            elif mapheight <= 64:
                vdpheight = 64
            elif mapheight <= 128:
                vdpheight = 128

        elif mapwidth <= 64:
            vdpwidth = 64

            if mapheight <= 32:
                vdpheight = 32
            elif mapheight <= 64:
                vdpheight = 64

        elif mapwidth <= 128:
            vdpwidth = 128

            if mapheight <= 32:
                vdpheight = 32


    else:
        vdpwidth = 64
        vdpheight = 32

        if (mapwidth > 128 and mapheight > 64):
            maptype = 'MAPTYPE_8WAY'
        elif mapwidth > 128 and mapheight <= 64:
            maptype = 'MAPTYPE_HORIZONTAL'
            if mapheight > 32:
                vdpheight = 64;
        elif mapwidth <= 128:
            maptype = 'MAPTYPE_VERTICAL'
            if mapwidth > 64:
                vdpwidth = 128
            elif mapwidth <= 32:
                vdpwidth = 32



def loadAssetListData():
    global vpdassetlistdict

    assetlistfilenames = glob('..\gamedata\VDPAssetList\*.vdpassetlist')
    for assetlistfilename in assetlistfilenames:
        objecttypename = os.path.splitext(assetlistfilename)[0]
        objecttypename = os.path.basename(objecttypename)

        assetlist = []

        with open(assetlistfilename) as f:
            for line in f:
                line = line.strip()
                assetlist.append(line)

        vpdassetlistdict[objecttypename] = assetlist

def exportc_vdpassetfunction(outputmapname, exported_c):

    exported_c.write('void ' + outputmapname + '_LoadVDPAssets(const VPMap* thismap)\n{\n')

    if objectgroup is not None:
        global vpdassetlistdict

        exportedobjecttypes = set();
        for child in objectgroup:
            objectid = child.get('objectid')
            if objectid is not None:
                objectid = int(objectid)
                exportedobjecttypes.add(child.get('objecttype'))

        exportedassets = set()

        for objecttype in exportedobjecttypes:
            if objecttype in vpdassetlistdict:
                vdpassetlist = vpdassetlistdict[objecttype]
                for vdpasset in vdpassetlist:
                    exportedassets.add(vdpasset)

        exportedassets = sorted(exportedassets)

        for exportedasset in exportedassets:
            exported_c.write('    LOAD_GGANIMATION_TO_VDP(' + exportedasset + ');\n')

    exported_c.write('''\n    VDPManager_LoadSpriteDataToVDP(thismap->tilesetSprite, 
                                   thismap->tilesetVDPInfo);\n\n''')

    exported_c.write('}\n\n')

def exportc_objectspawnlist(outputmapname, exported_c):
    global playerStartX
    global playerStartY

    exported_c.write('SpawnStruct const ' + outputmapname + '_ObjectSpawns[] =\n')
    exported_c.write('{\n')
    counter = 0

    for child in objectgroup:
        objectid = child.get('objectid')
        if objectid is not None:
            yposition = child.get('y')
            xposition = child.get('x')
            objecttype = child.get('objecttype')
            if objecttype == 'Player':
                playerStartX = xposition;
                playerStartY = yposition;
            else:
                exported_c.write('    {' + str(counter) + ', ' + objecttype + 'Create, PIXEL_TO_VIRTUAL(' + xposition + '), PIXEL_TO_VIRTUAL(' + yposition + ')},\n')
                counter = counter + 1
    exported_c.write('};\n\n')

def exportc_objectspawnsectionlists(outputmapname, exported_c):
    global mapwidth
    global sections
    global sections_count

    exported_c.write('const SpawnStruct* const ' + outputmapname + '_ObjectSpawnLists[] =\n')
    exported_c.write('{\n')

    counter = 0

    for child in objectgroup:
        objectid = child.get('objectid')
        if objectid is not None:
            objecttype = child.get('objecttype')
            if objecttype != 'Player':
                if maptype == 'MAPTYPE_HORIZONTAL' or maptype == 'MAPTYPE_VDP':
                    xposition = child.get('x')

                    section_left_xposition = int(xposition) / SECTION_PIXEL_SIZE
                    if section_left_xposition < 0:
                        section_left_xposition = 0
                    if section_left_xposition > (mapwidth / SECTION_TILE_SIZE) - 1:
                        section_left_xposition = (mapwidth / SECTION_TILE_SIZE) -1

                    if section_left_xposition not in sections:
                        sections[section_left_xposition] = set()
                    sections[section_left_xposition].add(counter)

                    width = int(child.get('width'))
                    section_right_xposition = (int(xposition) + (width - 1))  / SECTION_PIXEL_SIZE
                    if section_right_xposition < 0:
                        section_right_xposition = 0
                    if section_right_xposition > mapwidth / SECTION_TILE_SIZE:
                        section_right_xposition = mapwidth / SECTION_TILE_SIZE

                    if section_right_xposition not in sections:
                        sections[section_right_xposition] = set()
                    sections[section_right_xposition].add(counter)
                elif maptype == 'MAPTYPE_VERTICAL':
                    yposition = child.get('y')

                    section_up_yposition = int(yposition) / SECTION_PIXEL_SIZE
                    if section_up_yposition < 0:
                        section_up_yposition = 0
                    if section_up_yposition > (mapheight / SECTION_TILE_SIZE) - 1:
                        section_up_yposition = (mapheight / SECTION_TILE_SIZE) -1

                    if section_up_yposition not in sections:
                        sections[section_up_yposition] = set()
                    sections[section_up_yposition].add(counter)

                    height = int(child.get('height'))
                    section_down_yposition = (int(yposition) + (height - 1))  / SECTION_PIXEL_SIZE
                    if section_down_yposition < 0:
                        section_down_yposition = 0
                    if section_down_yposition > mapheight / SECTION_TILE_SIZE:
                        section_down_yposition = mapheight / SECTION_TILE_SIZE

                    if section_down_yposition not in sections:
                        sections[section_down_yposition] = set()
                    sections[section_down_yposition].add(counter)
                

                counter = counter + 1

    for loop in range(0, sections_count):
        if loop in sections:
            list = sections[loop]
            for item in list:
                exported_c.write('    &' + outputmapname + '_ObjectSpawns[' + str(item)  + '],\n')
            exported_c.write('    NULL,\n')

    exported_c.write('};\n\n')

def exportc_objectspawnsectionarrays(outputmapname, exported_c):
    global mapwidth
    global sections
    global sections_count 

    exported_c.write('SpawnList const ' + outputmapname + '_ObjectSpawnSections[' + str(sections_count) + '] =\n')
    exported_c.write('{\n')


    accumulator = 0

    for loop in range(0, sections_count):
        if loop in sections:
            list = sections[loop]
            exported_c.write('    ' + outputmapname + '_ObjectSpawnLists + ' + str(accumulator) + ',\n')
            accumulator = accumulator + len(list) + 1
        else:
            exported_c.write('    NULL,\n')

    exported_c.write('};\n\n')

def exportc_objectspawnsections(outputmapname, exported_c):
    global sections_count
    global sections
    global playerStartX
    global playerStartY

    if objectgroup is None:
        return

    sections = {}

    if maptype == 'MAPTYPE_HORIZONTAL' or maptype == 'MAPTYPE_VDP':
        sections_count = mapwidth / SECTION_TILE_SIZE
    if maptype == 'MAPTYPE_VERTICAL':
        sections_count = mapheight / SECTION_TILE_SIZE

    #print 'sections_count' + str(sections_count)
    #print 'mapwidth' + str(mapwidth)

    exportc_objectspawnlist(outputmapname, exported_c)
    exportc_objectspawnsectionlists(outputmapname, exported_c)
    exportc_objectspawnsectionarrays(outputmapname, exported_c)

def exportcfile(outputname, destinationfilename):
    global mapwidth
    global mapheight
    global maptype
    global vdpwidth
    global vdpheight
    global tilemapdata
    global objectgroup
    global tilesetfilename
    global sections_count
    global playerStartX
    global playerStartY

    exported_c = open(destinationfilename, "w")
    exported_c.write("// **************************************************** \n")
    exported_c.write("// This file is automatically generated by\n")
    exported_c.write("// export_tiled_maps.py\n")
    exported_c.write("// **************************************************** \n\n")    
    
    exported_c.write('#include "' + outputname + '.h"\n\n')
    exported_c.write('#include "..\..\MathUtils.h"\n')
    exported_c.write('#include "..\..\VDPManager.h"\n')
    exported_c.write('#include "..\..\PlayerManager.h"\n')
    exported_c.write('#include "..\..\MathUtils.h"\n')
    exported_c.write('#include "..\..\exported\ExportedGGAnimations.h"\n')
    exported_c.write('#include "..\..\exported\ExportedGenResSprites.h"\n')
    exported_c.write('#include "..\..\exported\ExportedTileAttributes.h"\n')
    exported_c.write('#include "..\..\generated\AllObjects.h"\n')
    exported_c.write('#include "..\..\VDPManager.h"\n')
    exported_c.write("\n")

    exported_c.write("unsigned short const " + outputname + "_tilemapdata[" + str(mapwidth * mapheight) + "] = // " + str(mapwidth) + " * " + str(mapheight) + "\n")
    exported_c.write("{\n")
    contents = ""
 
    for heightcounter in range(0, mapheight):
        for widthcounter in range(0, mapwidth):
            tile = int(tilemapdata[widthcounter + (heightcounter * mapwidth)])

            if tile > 0:
                tile = tile - 1
            contents = contents + str(tile)
            contents = contents + ","
        contents = contents + "\n"
    exported_c.write(contents)
    exported_c.write("\n};\n\n")

    playerStartX = 0
    playerStartY = 0

    exportc_vdpassetfunction(outputname, exported_c)
    exportc_objectspawnsections(outputname, exported_c)

    exported_c.write('const VPMap ' + outputname + ' = \n')
    exported_c.write('{\n')
    exported_c.write('    ' + outputname + '_LoadVDPAssets,\n')

    if objectgroup is None:
        exported_c.write('    NULL, // object spawn sections\n')
        exported_c.write('    0, // num object spawn sections\n')
    else:
        exported_c.write('    ' + outputname + '_ObjectSpawnSections,    // object spawn sections\n')
        exported_c.write('    ' + str(sections_count) + ',    // num object spawn sections\n')

    exported_c.write('    ' + str(mapwidth) + ',    // map width\n')
    exported_c.write('    ' + str(mapheight) + ',    // map height\n')

    exported_c.write('    ' + str(maptype) + ',    // map type\n')
    exported_c.write('    ' + str(vdpwidth) + ',    // vdp plane width\n')
    exported_c.write('    ' + str(vdpheight) + ',    // vdp plane height\n')

    exported_c.write('    ' + outputname + '_tilemapdata,\n')
    exported_c.write('    &' + tilesetfilename + 'Sprite,\n')
    exported_c.write('    &' + tilesetfilename + 'VDPInfo,\n')


    if objectgroup is None:
        exported_c.write('    NULL,\n')
    else:
        exported_c.write('    ' + tilesetfilename + 'TileAttributes,\n')
    
    exported_c.write('    PIXEL_TO_VIRTUAL(' + str(playerStartX) + '), // player start x\n')
    exported_c.write('    PIXEL_TO_VIRTUAL(' + str(playerStartY) + '), // player start y\n')
    exported_c.write('    ' + doorcontents + '\n')
    exported_c.write('}; \n\n')
    exported_c.close()

def exporthfile(outputname, destinationfilename):
    exported_h = open(destinationfilename, "w")

    exported_h.write("// **************************************************** \n")
    exported_h.write("// This file is automatically generated by\n")
    exported_h.write("// export_gentiled_maps.py\n")
    exported_h.write("// **************************************************** \n\n")
    exported_h.write('#ifndef ' + outputname.upper() + '_MAP_INCLUDE_H\n')
    exported_h.write('#define ' + outputname.upper() + '_MAP_INCLUDE_H\n\n')
    exported_h.write('#include "..\..\Map.h"\n\n')
    exported_h.write('extern const VPMap ' + outputname + ';\n')
    exported_h.write('\n#endif\n')
    exported_h.close()  



from glob import glob
print "************** Exporting Tiled Maps To Code **************"
filenames = glob('..\gamedata\Maps\*.gmx')
    
loadAssetListData()

includes=[]
    
for filename in filenames:
    sourcefilename = os.path.realpath('.') + "\\" + filename
    
    outputname = os.path.splitext(filename)[0]
    outputname = os.path.basename(outputname)   
    destinationcfilename = os.path.realpath('.') + '\\' + destination_folder + '\\' + outputname + '.c'
    destinationhfilename = os.path.realpath('.') + '\\' + destination_folder + '\\' + outputname + '.h'

    includes.append(outputname + ".h")
    
    docopy = False
    
    if (os.path.exists(destinationcfilename) == False):
        #print destinationcfilename + " is missing. Copying"
        docopy = True
    
    if docopy == False:
        sourcetime = os.path.getmtime(sourcefilename)
        destinationtime = os.path.getmtime(destinationcfilename)

        if sourcetime > destinationtime:
            #print "sourcefilename is newer than destination filename"
            docopy = True

    if docopy == True:
        print "Exporting " + filename + " to " + destination_folder

        if not os.path.exists(destination_path):
            os.makedirs(destination_path)
    
        importmap(filename)
        computevdpsettings()
        exportcfile(outputname, destinationcfilename)
        exporthfile(outputname, destinationhfilename)
    
    
exported_h = open("exported\\ExportedMaps.h", "w")
exported_h.write("// **************************************************** \n")
exported_h.write("// This file is automatically generated by\n")
exported_h.write("// export_gentiled_maps.py\n")
exported_h.write("// **************************************************** \n\n")
exported_h.write('#ifndef EXPORTED_GENTILED_MAPS_H\n')
exported_h.write('#define EXPORTED_GENTILED_MAPS_H\n\n')

for include in includes:
    exported_h.write('#include "maps\\' + include + '"\n')

exported_h.write('\n#endif\n')
exported_h.close()
    
